//-----------------------------------------------------------------------------
/*
Programmed by Olivier Sangala
http://olivier.sangala.free.fr
olivier_sangala@hotmail.com
*/
//-----------------------------------------------------------------------------

#include <tigcclib.h>
#include <extgraph.h>

//-----------------------------------------------------------------------------

enum {SHOCK_L,SHOCK_R,SHOCK_U,SHOCK_D};

//-----------------------------------------------------------------------------

void NewTry(void);
void CheckCursorPos(void);

void MoveBall(void);
void OnShock(int shock);
void XOR_Line_Cursor(void);
void MoveCursor(int key);

//-----------------------------------------------------------------------------

int curx=80;
int cury=50;
int posx;
int posy;
int basx;
int basy;

int posx0;
int posy0;
float v0x;
float v0y;
float t;
float t0;

float g=9.81;
float coeffr=0.8;


//-----------------------------------------------------------------------------

unsigned short cursor_data[16]={
0x0400,0x0400,0x0E00,0x1500,0x2480,0xFFE0,0x2480,0x1500,
0x0E00,0x0400,0x0400,0x0000,0x0000,0x0000,0x0000,0x0000,};

unsigned short basket_data[16] =
{
0x1FFF,0x0AAA,0x0AAA,0x0554,0x0554,0x02A8,0x02A8,0x0150, 
0x0150,0x00A0,0x00A0,0x0000,0x0000,0x0000,0x0000,0x0000, 
} ;


//-----------------------------------------------------------------------------

void NewTry(void)
{
	posx=random(130)+10; // 10-140
	posy=random(80)+10; // 10-90
	
	basx=160-16;
	basy=random(60)+20; // 20-80
	
	CheckCursorPos();
	
	ClrScr();
	Sprite16_XOR(basx,basy,12,basket_data,LCD_MEM);
	
	XOR_Line_Cursor();
	
}
//-----------------------------------------------------------------------------

void CheckCursorPos(void)
{
	if (curx<10)
	curx=10;
	if (curx>140)
	curx=140;
	if (cury<10)
	cury=10;
	if (cury>90)
	cury=90;
}
//-----------------------------------------------------------------------------

void MoveBall(void)
{
	
	v0x=curx-posx;
	v0y=cury-posy;
	t0=0;
	t=0;
	
	posx0=posx;
	posy0=posy;
	
	int x1=0,y1=0;
	int x2=0,y2=0;
	float t1=0,t2=0;
	
	int ball_in_goal=0;
	
	for (t=0;;t+=0.05)
	{
		if (((abs(v0x)<0.5 && v0x) || (abs(v0y)<10 && !v0x))  && posy==99)
		break;
		
		// POSITION FORMULA
		posx=posx0+v0x*(t-t0);
		posy=posy0+v0y*(t-t0)+g*(t-t0)*(t-t0)/2;
		
		
		// TEST FOR BASKET
		
		if (posy<basy)
		{
			x1=posx;
			y1=posy;
			t1=t;
		}
		else if (posy>=basy)
		{
			x2=posx;
			y2=posy;
			t2=t;
		}
		
		if (t1 && t2 && (t2-t1)==0.05)
		{
			float x=x1+(x2-x1)*(y1-basy)/(y1-y2);
			if (x>160-12)
			{
				ball_in_goal=1;
				printf_xy(0,0,"GOAL!");
			}
			
		}
		
		
		if (posx<0)
		OnShock(SHOCK_L);
		else if (posy<0)
		OnShock(SHOCK_U);
		else if (posx>159)
		OnShock(SHOCK_R);
		else if (posy>99)
		OnShock(SHOCK_D);

		EXT_SETPIX(LCD_MEM,posx,posy);
	}
	
	if (!ball_in_goal)
	printf_xy(0,0,"MISSED!");
	
	ngetchx();
}

//-----------------------------------------------------------------------------

void OnShock(int shock)
{
	if (shock==SHOCK_L)
	posx=0;
	else if (shock==SHOCK_U)
	posy=0;
	else if (shock==SHOCK_R)
	posx=159;
	else if (shock==SHOCK_D)
	posy=99;
	
	// SPEED FORMULA
	t-=0.05;
	float v0x2=v0x;
	float v0y2=v0y+g*(t-t0);
	t+=0.05;
	
	if (shock==SHOCK_U || shock==SHOCK_D)
	v0y2=-v0y2;
	else
	v0x2=-v0x2;
	
	v0x=v0x2*coeffr;
	v0y=v0y2*coeffr;
	
	posx0=posx;
	posy0=posy;
	t0=t;
	
	/*
	printf_xy(0,10,"%f",v0x);
	printf_xy(0,20,"%f",v0y);
	printf_xy(0,30,"%f",t);
	*/
	
	
}
//-----------------------------------------------------------------------------

void XOR_Line_Cursor(void)
{
	FastDrawLine(LCD_MEM,posx,posy,curx,cury,A_XOR);
	Sprite16_XOR(curx-5,cury-5,12,cursor_data,LCD_MEM);
}
//-----------------------------------------------------------------------------

void MoveCursor(int key)
{
			XOR_Line_Cursor();
			
			if (key==KEY_LEFT)
			curx--;
			else if (key==KEY_RIGHT)
			curx++;
			else if (key==KEY_UP)
			cury--;
			else if (key==KEY_DOWN)
			cury++;
			
			else if (key==KEY_LEFT+4096)
			curx-=10;
			else if (key==KEY_RIGHT+4096)
			curx+=10;
			else if (key==KEY_UP+4096)
			cury-=10;
			else if (key==KEY_DOWN+4096)
			cury+=10;
			
			CheckCursorPos();
			XOR_Line_Cursor();
	
}
//-----------------------------------------------------------------------------

// Main Function
void _main(void)
{

	clrscr();
	FontSetSys(0);
	printf("Basko 1.0 \n");
	printf("Programmed by Olivier Sangala \n");
	printf("http://olivier.sangala.free.fr \n");
	printf("olivier_sangala@hotmail.com \n");
	FontSetSys(1);
	ngetchx();
	
	randomize();
	
	NewTry();
	
	//EXT_SETPIX(LCD_MEM,posx,99-posy);
	
	for (;;)
	{
	
	
		int key=ngetchx();
		
		
		if (key==KEY_LEFT || key==KEY_RIGHT || key==KEY_UP || key==KEY_DOWN ||
		key==KEY_LEFT+4096 || key==KEY_RIGHT+4096 || key==KEY_UP+4096 || key==KEY_DOWN+4096)
		MoveCursor(key);
		
		
		else if (key==KEY_F1)
		{
		XOR_Line_Cursor();
		MoveBall();
		NewTry();
		}
		
		else if (key==KEY_ESC)
		break;
	}
	
}
